package de.rmdir.ms2debounce;

import java.io.InputStream;
import java.io.OutputStream;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.DataOutputStream;

import android.content.Context;
import android.content.SharedPreferences;

public class DebounceModuleHelper
{
	private Context ctx;
	public static final String PREFS_NAME = "DebounceCfg";
	final int SUPERUSER_REQUEST = 4223;

	public DebounceModuleHelper(Context context) {
		ctx = context;
	}

	public void loadModule() {
		_loadModule();
		setDelay(getSavedDelay());
	}

	protected void runAsRoot(String command) throws java.io.IOException,java.lang.InterruptedException {
		Process rootcmd = Runtime.getRuntime().exec(new String[]{"su","-c","sh"});
		DataOutputStream sh = new DataOutputStream(rootcmd.getOutputStream());
		sh.writeBytes(command + "\n");
		sh.writeBytes("exit\n");
		sh.flush();
		sh.close();

		rootcmd.waitFor();
	}

	public synchronized void _loadModule() {
		File debounce_ko = new File(ctx.getFilesDir() + "/debounce.ko");

		extractModule();

		SharedPreferences settings = ctx.getSharedPreferences(PREFS_NAME, Context.MODE_PRIVATE);
		SharedPreferences.Editor editor = settings.edit();
		if (is_safe_to_load()) {
			editor.putBoolean("safe_to_load", false);
			editor.commit();
		}

		try {
			runAsRoot("/system/bin/insmod " + debounce_ko);
		} catch (Exception e) {
			return;
		}

		if (!isLoaded()) {
			return;
		}

		if (getDelay() < 0) {
			return;
		}

		/* Module was obviously loaded, so it is safe to load on boot */
		editor.putBoolean("safe_to_load", true);
		editor.commit();
	}

	public synchronized void unloadModule() {
		try {
			runAsRoot("/system/bin/rmmod debounce");
		} catch (Exception e) {}
	}

	public synchronized boolean isLoaded() {
		boolean loaded = false;
		try {
			String read;

			FileReader modules = new FileReader("/proc/modules");
			BufferedReader modules_buf = new BufferedReader(modules);

			while((read = modules_buf.readLine()) != null) {
				if (read.regionMatches(0, "debounce", 0, 8)) {
					loaded = true;
				}
			}

		} catch (Exception e) {
			loaded = false;
		}

		return loaded;
	}

	public synchronized int getDelay() {
		int debounce_delay = -1;

		try {
			String read;

			FileReader delay = new FileReader("/sys/devices/debounce/debounce_delay");
			BufferedReader delay_buf = new BufferedReader(delay);

			read = delay_buf.readLine();
			if (read != null) {
				debounce_delay = Integer.parseInt(read.trim());
			}

			delay_buf.close();
		} catch (Exception e) {}

		return debounce_delay;
	}

	public synchronized void setDelay(int debounce_delay) {
		if (!isLoaded()) {
			return;
		}

		try {
			FileWriter delay = new FileWriter("/sys/devices/debounce/debounce_delay");
			BufferedWriter delay_buf = new BufferedWriter(delay);

			delay_buf.write((new Integer(debounce_delay)).toString());

			delay_buf.close();
		} catch (Exception e) {}
	}

	public synchronized int getSavedDelay() {
		SharedPreferences settings = ctx.getSharedPreferences(PREFS_NAME, Context.MODE_PRIVATE);

		return settings.getInt("debounce_delay", 8);
	}

	public synchronized void setSavedDelay(int delay) {
		SharedPreferences settings = ctx.getSharedPreferences(PREFS_NAME, Context.MODE_PRIVATE);
		SharedPreferences.Editor editor = settings.edit();

		editor.putInt("debounce_delay", delay);
		editor.commit();
	}

	public synchronized boolean is_safe_to_load() {
		SharedPreferences settings = ctx.getSharedPreferences(PREFS_NAME, Context.MODE_PRIVATE);
		return settings.getBoolean("safe_to_load", false);
	}

	public synchronized boolean get_on_boot() {
		SharedPreferences settings = ctx.getSharedPreferences(PREFS_NAME, Context.MODE_PRIVATE);
		return settings.getBoolean("on_boot", false);
	}

	public synchronized void set_on_boot(boolean on_boot) {
		SharedPreferences settings = ctx.getSharedPreferences(PREFS_NAME, Context.MODE_PRIVATE);
		SharedPreferences.Editor editor = settings.edit();

		editor.putBoolean("on_boot", on_boot);
		editor.commit();
	}

	private synchronized void extractModule() {
		File debounce_ko = new File(ctx.getFilesDir() + "/debounce.ko");

		if (debounce_ko.exists()) {
			return;
		}

		try {
			InputStream apk = ctx.getAssets().open("debounce.ko");
			OutputStream mod = ctx.openFileOutput("debounce.ko.tmp", 0);

			//I assume a page is 4k...
			byte buf[] = new byte[4096];
			int bytes;

			while((bytes = apk.read(buf)) != -1) {
				mod.write(buf, 0, bytes);
			}

			apk.close();
			mod.close();

			File tmpfile = new File(debounce_ko + ".tmp");
			tmpfile.renameTo(debounce_ko);
		} catch (Exception e) {}
	}
}
